local ADDON = ...

GMDURABILITY = {
     AUTOREPAIR = true,
     GUILDREPAIR = true,
     AUTOSELL = true,
}


local LibQTip = LibStub('LibQTip-1.0')
local ldb = LibStub:GetLibrary("LibDataBroker-1.1")
 
local dataobj = ldb:NewDataObject("gmDurability", {
    type = "data source",
    icon = "Interface\\Minimap\\Tracking\\Repair",
    text = "test",  
})

-- working vars
local prgname = "|cffffd200gmDurability|r"
local idx, perc
local format_color = "|cff%02x%02x%02x%s"
local durabilities = {}

local slots = {
	"HeadSlot",
	"ShoulderSlot",
		"BackSlot", -- no durability
	"ChestSlot",
	"WristSlot",
	"HandsSlot",
	"WaistSlot",
	"LegsSlot",
	"FeetSlot",
		"Finger0Slot", -- no durability
		"Finger1Slot", -- no durability
		"Trinket0Slot", -- no durability
		"Trinket1Slot", -- no durability		
	"MainHandSlot",
	"SecondaryHandSlot",
}

local text_color = {
	{0,1,0},
	{1,1,0},
	{1,0.5,0},
	{1,0,0},
	{1,1,1},   -- for "-" durabilities
}


local math_min = math.min
local string_format = string.format
local string_gsub = string.gsub
local table_insert = table.insert
local string_find = string.find
local tooltip

local function unescape(arg)
  local Result = tostring(arg)
  Result = gsub(Result, "|H.-|h(.-)|h", "%1") -- Remove links.
  Result = gsub(Result, "%[", "") -- Remove links.
  Result = gsub(Result, "%]", "") -- Remove links.
  return Result
end


local function colorperc(arg)
	
	local arg = tonumber(arg)
	
	if     arg >= 0  	and arg < 0.25  then return 4
	elseif arg >= 0.25 	and arg < 0.50  then return 3
	elseif arg >= 0.50 	and arg < 0.75  then return 2
	elseif arg >= 0.75 	and	arg <= 1 	then return 1
	end

end

local function anchor_OnEnter(self)

	local row, col
    local tooltip = LibQTip:Acquire(ADDON.."tip", 3, "LEFT", "LEFT", "RIGHT")
    self.tooltip = tooltip 
	tooltip:SmartAnchorTo(self)

	
	row = tooltip:AddHeader()
    tooltip:SetCell(row,1,ADDON,"LEFT",3)
	tooltip:SetCellTextColor(row,1,1,1,0)
	
 	row = tooltip:AddLine("")
	row = tooltip:AddLine()
	tooltip:SetCell(row,1,string.format("Item Level: Total %.1f - Equipped %.1f", select(1,GetAverageItemLevel()), select(2,GetAverageItemLevel())),"LEFT",3)	
	row = tooltip:AddLine("")	
	row = tooltip:AddLine("ilv","name","% durability")
	tooltip:SetLineTextColor(row,1,1,0)	
	tooltip:AddSeparator()
		
	for idx=1, #slots do
			if GetInventoryItemLink("player", GetInventorySlotInfo(slots[idx])) then 			
				
				local tempdurability
				
				if durabilities[idx] == "-" then				
					tempdurability = 5
				else
					tempdurability = colorperc(tonumber(durabilities[idx]/100))
				end
								
				row = tooltip:AddLine(
					GetDetailedItemLevelInfo(GetInventoryItemLink("player", GetInventorySlotInfo(slots[idx]))),
					unescape(GetInventoryItemLink("player",GetInventorySlotInfo(slots[idx]))),
					string_format(format_color, text_color[tempdurability][1]*255, text_color[tempdurability][2]*255,text_color[tempdurability][3]*255, durabilities[idx]).."|r"
				)
				
				if	GetDetailedItemLevelInfo(GetInventoryItemLink("player", GetInventorySlotInfo(slots[idx]))) <= select(2,GetAverageItemLevel()) then 
				
					tooltip:SetCellTextColor(row,1,1,0,0)	
				
				else
				
					tooltip:SetCellTextColor(row,1,0,1,0)	
				
				end
				
				
				
			end
	end
	
	
	row = tooltip:AddLine(" ")	
	row = tooltip:AddLine()
	tooltip:SetCell(row,1,"Repair","LEFT",2)

	if GMDURABILITY["AUTOREPAIR"] == true then

		tooltip:SetCell(row,3,"YES","RIGHT")
		tooltip:SetCellTextColor(row,3,0,1,0)

	else

		tooltip:SetCell(row,3,"NO","RIGHT")
		tooltip:SetCellTextColor(row,3,1,0,0)

	end
	
	
	
	row = tooltip:AddLine()
	tooltip:SetCell(row,1,"Use guild funds","LEFT",2)
	
	if GMDURABILITY["GUILDREPAIR"] == true then 
	
		tooltip:SetCell(row,3,"YES","RIGHT")
		tooltip:SetCellTextColor(row,3,0,1,0)

	else

		tooltip:SetCell(row,3,"NO","RIGHT")
		tooltip:SetCellTextColor(row,3,1,0,0)

	end
	
	row = tooltip:AddLine()
	tooltip:SetCell(row,1,"Sell junk","LEFT",2)


	if GMDURABILITY["AUTOSELL"] == true then 
	
		tooltip:SetCell(row,3,"YES","RIGHT")
		tooltip:SetCellTextColor(row,3,0,1,0)

	else

		tooltip:SetCell(row,3,"NO","RIGHT")
		tooltip:SetCellTextColor(row,3,1,0,0)

	end
	
	row = tooltip:AddLine("")
	
	row = tooltip:AddLine()
	tooltip:SetCell(row,1,"Left Click","LEFT",2)
	tooltip:SetCell(row,3,"Toggle Repair","RIGHT")	

	row = tooltip:AddLine()
	tooltip:SetCell(row,1,"Middle Click","LEFT",2)
	tooltip:SetCell(row,3,"Toggle Guild funds","RIGHT")	

	row = tooltip:AddLine()
	tooltip:SetCell(row,1,"Right Click","LEFT",2)
	tooltip:SetCell(row,3,"Toggle Sell","RIGHT")	
	
	
	row = tooltip:AddLine(" ")
	row = tooltip:AddLine(" ")		
	tooltip:SetCell(row,1,"Legenda","LEFT",3)
	tooltip:SetCellTextColor(row,1,1,1,0)

	row = tooltip:AddLine()
	tooltip:SetCell(row,1,"ItemLevel: " .. string_format(format_color,255,0,0,"below").."|r / " .. string_format(format_color,0,255,0,"above").."|r average equipped","LEFT",3)

	row = tooltip:AddLine()
	tooltip:SetCell(row,1,"Durability: " .. 
	string_format(format_color,255,0,0,"0-24")..	"|r ".. 
	string_format(format_color,255,128,0,"25-49")..	"|r " ..
	string_format(format_color,255,255,0,"50-74")..	"|r " ..
	string_format(format_color,0,255,0,"75-100")..	"|r","LEFT",3)
	
	
    -- Show it, et voilà !
    row = tooltip:Show()

 
end
 
local function anchor_OnLeave(self)
   
   -- Release the tooltip
   LibQTip:Release(self.tooltip)
   self.tooltip = nil
   
end



local frame = CreateFrame("Frame")
frame:RegisterEvent("UPDATE_INVENTORY_DURABILITY")
frame:RegisterEvent("MERCHANT_SHOW")
frame:SetScript("OnEvent", function(self, event, ...)
	
	if event == "UPDATE_INVENTORY_DURABILITY" then
		
				local mindurability = 1
				durabilities = {}
				
				for idx=1, #slots do
					local slotid = GetInventorySlotInfo(slots[idx])
					local durability, maxdurability = GetInventoryItemDurability(slotid)  
															
					if durability and (durability and maxdurability and maxdurability ~= 0) then	
						table_insert(durabilities, tostring(string_format("%d", (durability/maxdurability)*100)))
						mindurability = math_min(durability/maxdurability, mindurability)		  
					else
						table_insert(durabilities, "-")
					end
				end	
					
				dataobj.text = string_format(format_color, text_color[colorperc(mindurability)][1]*255, text_color[colorperc(mindurability)][2]*255, text_color[colorperc(mindurability)][3]*255, string_format("%d%%",mindurability*100)).."|r"
				
				dataobj.OnClick = function(self, button)  

					if 		button == "LeftButton" 		then GMDURABILITY["AUTOREPAIR"] 	= not GMDURABILITY["AUTOREPAIR"] 					
					elseif 	button == "MiddleButton" 	then GMDURABILITY["GUILDREPAIR"] 	= not GMDURABILITY["GUILDREPAIR"] 					
					elseif 	button == "RightButton" 	then GMDURABILITY["AUTOSELL"] 		= not GMDURABILITY["AUTOSELL"] 
					end
					
				end
								
				dataobj.OnEnter = function(self)
					anchor_OnEnter(self)
				end
 
				dataobj.OnLeave = function(self)
					anchor_OnLeave(self) 
				end
					
	elseif event =="MERCHANT_SHOW" then 
	
				local mymoney = GetMoney()

				-- sell greys
				if GMDURABILITY["AUTOSELL"] == true then 
					local bag, slot
					for bag = 0,4,1 do 
						for slot = 1, GetContainerNumSlots(bag), 1 do 
							local name = GetContainerItemLink(bag,slot)
							if name and string_find(name,"ff9d9d9d") then 
								
								local _, itemCount = GetContainerItemInfo(bag, slot);
								local _, _, _, _, _, _, _, _, _, _, vendorPrice = GetItemInfo(name)
								UseContainerItem(bag,slot) 
								
								print(string_format("%s: selling %s %s for %s", prgname, itemCount, name, GetCoinTextureString(vendorPrice * itemCount)))
							end
						end
					end
				end
				
				-- autorepair
				local cost = GetRepairAllCost()	
				if cost > 0 and CanMerchantRepair() and GMDURABILITY["AUTOREPAIR"] then	
							
					if CanWithdrawGuildBankMoney() and CanGuildBankRepair() and GMDURABILITY["GUILDREPAIR"] == true then
						RepairAllItems(1)
						print(prgname .. ": repair with guild funds " .. GetCoinTextureString(cost))
					else
						if mymoney >= cost then
							RepairAllItems()
							print(prgname .. ": repair " .. GetCoinTextureString(cost))
						else 
							print(prgname .. ": no repair, need more money")
						end
					end
				end	
	
	
	end
end)
